// Design: code
// Description: Digital code key
// Author: Jorge Juan <jjchico@dte.us.es>
// Copyright Universidad de Sevilla, Spain
// Date: 14-12-2011

////////////////////////////////////////////////////////////////////////////////                                                                         //
// This file is free software: you can redistribute it and/or modify it under //
// the terms of the GNU General Public License as published by the Free       //
// Software Foundation, either version 3 of the License, or (at your option)  //
// any later version. You can access a copy of the GNU General Public License //
// at <http://www.gnu.org/licenses/>.                                         //
////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1ps

// Test bench

module test();

	reg clk = 1;		// clock
	reg reset = 0;		// reset
	reg [3:0] b;		// input
	wire z;			// output

	// Module instantiation
	code uut (.clk(clk), .reset(reset), .b(b), .z(z));

	initial begin
		// Waveform generation
		$dumpfile("test.vcd");
		$dumpvars(0, test);

		// Initialization
		#5	reset = 1;
			b = 4'b0000;
		#10	reset = 0;
		#15	;
		
		// Input data
		#20	b = 4'b0001;	// '0' pressed
		#20	b = 4'b0001;
		#20	b = 4'b0001;
		#20	b = 4'b0000;	// '0' released
		#20	b = 4'b0000;
		#20	b = 4'b0100;	// '2' pressed
		#20	b = 4'b0100;
		#20	b = 4'b0100;
		#20	b = 4'b0100;
		#20	b = 4'b0000;	// '2' released
		#20	b = 4'b0000;
		#20	b = 4'b0100;	// '2' pressed (bounce)
		#20	b = 4'b0100;
		#20	b = 4'b0000;	// '2' released
		#20	b = 4'b0000;
		#20	b = 4'b0000;
		#20	b = 4'b0000;
		#20	b = 4'b0010;	// '1' pressed (door should open)
		#20	b = 4'b0010;
		#20	b = 4'b0010;
		#20	b = 4'b0010;
		#20	b = 4'b0000;	// '1' released
		#20	b = 4'b0000;
		#20	b = 4'b1000;	// '3' pressed (door should close)
		#20	b = 4'b1000;
		#20	b = 4'b1000;
		#20	b = 4'b0000;	// '3' released
		#20	b = 4'b0000;
		#20	b = 4'b0100;	// some incorrect codes
		#20	b = 4'b0100;
		#20	b = 4'b0100;
		#20	b = 4'b0000;
		#20	b = 4'b0010;
		#20	b = 4'b0010;
		#20	b = 4'b0010;
		#20	b = 4'b0000;
		#20	b = 4'b0000;

		#40	$finish;
	end

	// Clock generator (50 MHz)
	always
		#10	clk = ~clk;
endmodule // test
